#############################################
# Analyse "BRUSSELS" artificial data
# Example code file
# For comparison with analysis of PARIS data 
# in Chapter 12
#############################################

-------------------------------------------------
# Getting started

# Ensure you have the following packages installed:
# use e.g. install.packages("meta") if not

library(meta) # meta-analysis
library(lme4) # generalized linear mixed effect models
library(dplyr) # data manipulation
library(tidyr) # data manipulation
library(purrr) # iteration / multiple analysis

# set directory to where data are stored
setwd("[INSERT PATH TO DATA DIRECTORY]")

# load BRUSSELS data
brussels <- read.csv("Brussels artificial data.csv")

# edit treatment data
brussels$Treatcode <- ifelse(brussels$Treatment=="Aspirin", 1, 0)

# --------------------------------------------------
# Two-stage meta-analysis of pre-eclampsia

# use dplyr/tidyr to generate summary data by trial
# [other methods are possible (e.g. using tapply)]

brussels.summary <- brussels %>%
  group_by(Trial, Treatment) %>%
  summarise(N=n(),
            N.preec=sum(Preeclampsia)) %>%
  pivot_wider(id_cols=Trial, names_from=Treatment, values_from=N:N.preec)

# meta-analysis (using metabin from meta library)
# gives both FE and RE results

ma.twostage <- metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                             data=brussels.summary, sm="OR", method="Inverse")

# forest plot
forest(ma.twostage, studlab=Trial)

#---------------------------------------------
# One-stage meta-analysis of pre-eclampsia

# FE
ma.onestage.fe <- glm(Preeclampsia ~ Trial + factor(Treatcode), 
                      data=brussels, family=binomial)
summary(ma.onestage.fe)

# RE (several options)
# using glmer from lme4 library

# random treatment, but independent trial effects
ma.onestage.re.1 <- glmer(Preeclampsia ~ Trial + factor(Treatcode) + (Treatcode|Trial), 
                          data=brussels, family=binomial, nAGQ=0)
summary(ma.onestage.re.1)

# random treatment and trial effects (correlated)
ma.onestage.re.2 <- glmer(Preeclampsia ~ factor(Treatcode) + (1+Treatcode|Trial), 
                          data=brussels, family=binomial)
summary(ma.onestage.re.2)

# random treatment and trial effects (uncorrelated)
ma.onestage.re.3 <- glmer(Preeclampsia ~ factor(Treatcode) + (1|Trial)+(Treatcode|Trial), 
                          data=brussels, family=binomial)
summary(ma.onestage.re.3)

#-----------------------------------------------
# convert log odds ratios to odds ratios 
# for ease of interpretation

log_convert <- function(result, level=0.95){
  coeffs <- summary(result)$coefficients
  wald.val <- -qnorm((1-level)/2)
  out <- data.frame(Estimate=exp(coeffs[,1]),
                    CIlow=exp(coeffs[,1]-wald.val*coeffs[,2]),
                    CIhigh=exp(coeffs[,1]+wald.val*coeffs[,2]))
  out <- round(out,3)
  return(out)
}

# FE and RE results as OR and 95% CI
log_convert(ma.onestage.fe)
log_convert(ma.onestage.re.2)


###########################################

#-----------------------------------------
# Analyses of covariates and interactions

#------------------------------------------
# One-stage analyses of treatment-covariate interaction

# Only RE models shown, 
# (without random effects on covariate or interaction)

# Diabetes
ma.onestage.diabetes <- glmer(Preeclampsia ~ factor(Treatcode)*factor(Diabetes) + (1+Treatcode|Trial), 
                              data=brussels, family=binomial)
summary(ma.onestage.diabetes)
log_convert(ma.onestage.diabetes)

# Hypertension
ma.onestage.hyper <- glmer(Preeclampsia ~ factor(Treatcode)*factor(Hypertension) + (1+Treatcode|Trial), 
                           data=brussels, family=binomial)
summary(ma.onestage.hyper)
log_convert(ma.onestage.hyper)

# Gestational age
ma.onestage.gestage <- glmer(Preeclampsia ~ factor(Treatcode)*GestationalAge + (1+Treatcode|Trial), 
                             data=brussels, family=binomial)
summary(ma.onestage.gestage)
log_convert(ma.onestage.gestage)

# Maternal age
ma.onestage.matage <- glmer(Preeclampsia ~ factor(Treatcode)*MaternalAge + (1+Treatcode|Trial), 
                            data=brussels, family=binomial)
summary(ma.onestage.matage)
log_convert(ma.onestage.matage)

# Gestational age and maternal age with centering of variables

# create centered variables
brussels.cent <- brussels %>%
  mutate(GAge.center=GestationalAge - mean(GestationalAge,na.rm=T),
         MAge.center=MaternalAge - mean(MaternalAge,na.rm=T))

# analyses

ma.gestage.cent <- glmer(Preeclampsia ~ factor(Treatcode)*GAge.center + (1+Treatcode|Trial), 
                         data=brussels.cent, family=binomial)
summary(ma.gestage.cent)
log_convert(ma.gestage.cent)

ma.matage.cent <- glmer(Preeclampsia ~ factor(Treatcode)*MAge.center + (1+Treatcode|Trial), 
                        data=brussels.cent, family=binomial)
summary(ma.matage.cent)
log_convert(ma.matage.cent)

#------------------------------------------------------
# Two-stage subgroup analyses of treatment-covariate interactions

# Diabetes

# summarise IPD

diabetes.summary <- brussels %>%
  group_by(Trial, Treatment, Diabetes) %>%
  summarise(N=n(),
            N.preec=sum(Preeclampsia)) %>%
  pivot_wider(id_cols=c(Trial,Diabetes), names_from=Treatment, values_from=N:N.preec)

# MA in people WITH diabetes
# (remove trials with no data first)

diab.with <- filter(diabetes.summary,Diabetes==1) %>%
  filter(!is.na(N_Aspirin), N.preec_Aspirin+N.preec_Placebo>1)

ma.with <- metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                   data=diab.with, sm="OR")

# MA in people WITHOUT diabetes

diab.without <- filter(diabetes.summary,Diabetes==0)

ma.without <- metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                   data=diab.without, sm="OR")

# An alternative succinct method using group_map() function

diabetes.subgroup.ma <- diabetes.summary %>%
  filter(!is.na(N_Aspirin), N.preec_Aspirin+N.preec_Placebo>1) %>%
  group_by(Diabetes) %>%
  group_map(~metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                    data=.x, sm="OR"))


# For hypertension use code as above 

# Gestational age

# summarise IPD, add a variable for gest. age < 20 weeks

gestage.summary <- brussels %>% 
  mutate(LowGest=ifelse(GestationalAge<20, 1, 0)) %>%
  group_by(Trial, Treatment, LowGest) %>%
  summarise(N=n(),
            N.preec=sum(Preeclampsia)) %>%
  pivot_wider(id_cols=c(Trial,LowGest), names_from=Treatment, values_from=N:N.preec)

# MA in people with GestAge <20 weeks
# (remove trials with no data first)

ga.with <- filter(gestage.summary,LowGest==1)

ma.with <- metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                   data=ga.with, sm="OR")

# MA in people with GestAge >=20 weeks

ga.without <- filter(gestage.summary,LowGest==0)

ma.without <- metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                      data=ga.without, sm="OR")

# or in more succinct code:

gestage.subgroup.ma <- gestage.summary %>%
  group_by(LowGest) %>%
  group_map(~metabin(N.preec_Aspirin, N_Aspirin, N.preec_Placebo, N_Placebo,
                     data=.x, sm="OR"))

# For maternal age adapt the code for gestational age


#--------------------------------------------------------
# Two-stage analyses of treatment-covariate interactions

# Meta-analysis of interactions approach 
# [Described in Chapter 12 Section 12.4.1, but no results given]

# function to analyse interaction in a single trial

ma_interaction <- function(trialname, covariate){
  
  tdata <- filter(brussels,Trial==trialname)
  covar <- tdata[[covariate]]
  trial.glm <- glm(Preeclampsia ~ factor(Treatcode)*covar, data=tdata, family=binomial)
  glm.results <- summary(trial.glm)$coefficients
  out <- data.frame(LOR=glm.results[,1], seLOR=glm.results[,2])
  return(out)
}

# use map functions in purrr library to easily do analysis for each trial

# Gestational age as an example
# fit regression model with interactions for each trial
# use map function from purr to aid this

GestAge.bytrial <- map_dfr(unique(brussels$Trial), 
                           ~ma_interaction(.x, "GestationalAge"))

# extract just the interaction results
GestAge.inter <- GestAge.bytrial[seq(4,96,4),]

# meta-analysis of interaction results
GestAge.inter.ma <- metagen(LOR, seLOR, data=GestAge.inter)

forest(GestAge.inter.ma)

# Use the same approach for other covariates
# NB: It is more complex for diabetes because some studies 
# have few people with diabetes, so interaction is inestimable.

##########################################################